// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure.Management.Automation;
using Microsoft.Azure.Management.Automation.Models;

namespace Microsoft.Azure.Management.Automation
{
    public static partial class DscCompilationJobOperationsExtensions
    {
        /// <summary>
        /// Creates the Dsc compilation job of the configuration.  (see
        /// http://aka.ms/azureautomationsdk/dscconfigurationcompilejoboperations
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Automation.IDscCompilationJobOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group
        /// </param>
        /// <param name='automationAccount'>
        /// Required. The automation account name.
        /// </param>
        /// <param name='parameters'>
        /// Required. The parameters supplied to the create compilation job
        /// operation.
        /// </param>
        /// <returns>
        /// The response model for the create Dsc Compilation job operation.
        /// </returns>
        public static DscCompilationJobCreateResponse Create(this IDscCompilationJobOperations operations, string resourceGroupName, string automationAccount, DscCompilationJobCreateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDscCompilationJobOperations)s).CreateAsync(resourceGroupName, automationAccount, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Creates the Dsc compilation job of the configuration.  (see
        /// http://aka.ms/azureautomationsdk/dscconfigurationcompilejoboperations
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Automation.IDscCompilationJobOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group
        /// </param>
        /// <param name='automationAccount'>
        /// Required. The automation account name.
        /// </param>
        /// <param name='parameters'>
        /// Required. The parameters supplied to the create compilation job
        /// operation.
        /// </param>
        /// <returns>
        /// The response model for the create Dsc Compilation job operation.
        /// </returns>
        public static Task<DscCompilationJobCreateResponse> CreateAsync(this IDscCompilationJobOperations operations, string resourceGroupName, string automationAccount, DscCompilationJobCreateParameters parameters)
        {
            return operations.CreateAsync(resourceGroupName, automationAccount, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Retrieve the Dsc configuration compilation job identified by job
        /// id.  (see
        /// http://aka.ms/azureautomationsdk/dsccompilationjoboperations for
        /// more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Automation.IDscCompilationJobOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group
        /// </param>
        /// <param name='automationAccount'>
        /// Required. The automation account name.
        /// </param>
        /// <param name='jobId'>
        /// Required. The Dsc configuration compilation job id.
        /// </param>
        /// <returns>
        /// The response model for the get Dsc compilation job operation.
        /// </returns>
        public static DscCompilationJobGetResponse Get(this IDscCompilationJobOperations operations, string resourceGroupName, string automationAccount, Guid jobId)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDscCompilationJobOperations)s).GetAsync(resourceGroupName, automationAccount, jobId);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Retrieve the Dsc configuration compilation job identified by job
        /// id.  (see
        /// http://aka.ms/azureautomationsdk/dsccompilationjoboperations for
        /// more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Automation.IDscCompilationJobOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group
        /// </param>
        /// <param name='automationAccount'>
        /// Required. The automation account name.
        /// </param>
        /// <param name='jobId'>
        /// Required. The Dsc configuration compilation job id.
        /// </param>
        /// <returns>
        /// The response model for the get Dsc compilation job operation.
        /// </returns>
        public static Task<DscCompilationJobGetResponse> GetAsync(this IDscCompilationJobOperations operations, string resourceGroupName, string automationAccount, Guid jobId)
        {
            return operations.GetAsync(resourceGroupName, automationAccount, jobId, CancellationToken.None);
        }
        
        /// <summary>
        /// Retrieve the job stream identified by job stream id.  (see
        /// http://aka.ms/azureautomationsdk/jobstreamoperations for more
        /// information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Automation.IDscCompilationJobOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group
        /// </param>
        /// <param name='automationAccount'>
        /// Required. The automation account name.
        /// </param>
        /// <param name='jobId'>
        /// Required. The job id.
        /// </param>
        /// <param name='jobStreamId'>
        /// Required. The job stream id.
        /// </param>
        /// <returns>
        /// The response model for the get job stream operation.
        /// </returns>
        public static JobStreamGetResponse GetOutput(this IDscCompilationJobOperations operations, string resourceGroupName, string automationAccount, Guid jobId, string jobStreamId)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDscCompilationJobOperations)s).GetOutputAsync(resourceGroupName, automationAccount, jobId, jobStreamId);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Retrieve the job stream identified by job stream id.  (see
        /// http://aka.ms/azureautomationsdk/jobstreamoperations for more
        /// information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Automation.IDscCompilationJobOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group
        /// </param>
        /// <param name='automationAccount'>
        /// Required. The automation account name.
        /// </param>
        /// <param name='jobId'>
        /// Required. The job id.
        /// </param>
        /// <param name='jobStreamId'>
        /// Required. The job stream id.
        /// </param>
        /// <returns>
        /// The response model for the get job stream operation.
        /// </returns>
        public static Task<JobStreamGetResponse> GetOutputAsync(this IDscCompilationJobOperations operations, string resourceGroupName, string automationAccount, Guid jobId, string jobStreamId)
        {
            return operations.GetOutputAsync(resourceGroupName, automationAccount, jobId, jobStreamId, CancellationToken.None);
        }
        
        /// <summary>
        /// Retrieve a list of dsc compilation jobs.  (see
        /// http://aka.ms/azureautomationsdk/compilationjoboperations for more
        /// information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Automation.IDscCompilationJobOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group
        /// </param>
        /// <param name='automationAccount'>
        /// Required. The automation account name.
        /// </param>
        /// <param name='parameters'>
        /// Optional. The parameters supplied to the list operation.
        /// </param>
        /// <returns>
        /// The response model for the list job operation.
        /// </returns>
        public static DscCompilationJobListResponse List(this IDscCompilationJobOperations operations, string resourceGroupName, string automationAccount, DscCompilationJobListParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDscCompilationJobOperations)s).ListAsync(resourceGroupName, automationAccount, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Retrieve a list of dsc compilation jobs.  (see
        /// http://aka.ms/azureautomationsdk/compilationjoboperations for more
        /// information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Automation.IDscCompilationJobOperations.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group
        /// </param>
        /// <param name='automationAccount'>
        /// Required. The automation account name.
        /// </param>
        /// <param name='parameters'>
        /// Optional. The parameters supplied to the list operation.
        /// </param>
        /// <returns>
        /// The response model for the list job operation.
        /// </returns>
        public static Task<DscCompilationJobListResponse> ListAsync(this IDscCompilationJobOperations operations, string resourceGroupName, string automationAccount, DscCompilationJobListParameters parameters)
        {
            return operations.ListAsync(resourceGroupName, automationAccount, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Retrieve next list of dsc compilation jobs.  (see
        /// http://aka.ms/azureautomationsdk/compilationjoboperations for more
        /// information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Automation.IDscCompilationJobOperations.
        /// </param>
        /// <param name='nextLink'>
        /// Required. The link to retrieve next set of items.
        /// </param>
        /// <returns>
        /// The response model for the list job operation.
        /// </returns>
        public static DscCompilationJobListResponse ListNext(this IDscCompilationJobOperations operations, string nextLink)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IDscCompilationJobOperations)s).ListNextAsync(nextLink);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Retrieve next list of dsc compilation jobs.  (see
        /// http://aka.ms/azureautomationsdk/compilationjoboperations for more
        /// information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.Azure.Management.Automation.IDscCompilationJobOperations.
        /// </param>
        /// <param name='nextLink'>
        /// Required. The link to retrieve next set of items.
        /// </param>
        /// <returns>
        /// The response model for the list job operation.
        /// </returns>
        public static Task<DscCompilationJobListResponse> ListNextAsync(this IDscCompilationJobOperations operations, string nextLink)
        {
            return operations.ListNextAsync(nextLink, CancellationToken.None);
        }
    }
}
