// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for
// license information.
// 
// Code generated by Microsoft (R) AutoRest Code Generator 0.16.0.0
// Changes may cause incorrect behavior and will be lost if the code is
// regenerated.

namespace Microsoft.Azure.Management.Cdn
{
    using System;
    using System.Collections.Generic;
    using System.Net.Http;
    using System.Threading;
    using System.Threading.Tasks;
    using Microsoft.Rest;
    using Microsoft.Rest.Azure;
    using Models;

    /// <summary>
    /// OriginsOperations operations.
    /// </summary>
    public partial interface IOriginsOperations
    {
        /// <summary>
        /// Lists the existing CDN origins within an endpoint.
        /// </summary>
        /// <param name='endpointName'>
        /// Name of the endpoint within the CDN profile.
        /// </param>
        /// <param name='profileName'>
        /// Name of the CDN profile within the resource group.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Name of the resource group within the Azure subscription.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        Task<AzureOperationResponse<IEnumerable<Origin>>> ListByEndpointWithHttpMessagesAsync(string endpointName, string profileName, string resourceGroupName, Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Gets an existing CDN origin within an endpoint.
        /// </summary>
        /// <param name='originName'>
        /// Name of the origin, an arbitrary value but it needs to be unique
        /// under endpoint
        /// </param>
        /// <param name='endpointName'>
        /// Name of the endpoint within the CDN profile.
        /// </param>
        /// <param name='profileName'>
        /// Name of the CDN profile within the resource group.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Name of the resource group within the Azure subscription.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        Task<AzureOperationResponse<Origin>> GetWithHttpMessagesAsync(string originName, string endpointName, string profileName, string resourceGroupName, Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Creates a new CDN origin within an endpoint.
        /// </summary>
        /// <param name='originName'>
        /// Name of the origin, an arbitrary value but it needs to be unique
        /// under endpoint
        /// </param>
        /// <param name='originProperties'>
        /// Origin properties
        /// </param>
        /// <param name='endpointName'>
        /// Name of the endpoint within the CDN profile.
        /// </param>
        /// <param name='profileName'>
        /// Name of the CDN profile within the resource group.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Name of the resource group within the Azure subscription.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        Task<AzureOperationResponse<Origin>> CreateWithHttpMessagesAsync(string originName, OriginParameters originProperties, string endpointName, string profileName, string resourceGroupName, Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Creates a new CDN origin within an endpoint.
        /// </summary>
        /// <param name='originName'>
        /// Name of the origin, an arbitrary value but it needs to be unique
        /// under endpoint
        /// </param>
        /// <param name='originProperties'>
        /// Origin properties
        /// </param>
        /// <param name='endpointName'>
        /// Name of the endpoint within the CDN profile.
        /// </param>
        /// <param name='profileName'>
        /// Name of the CDN profile within the resource group.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Name of the resource group within the Azure subscription.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        Task<AzureOperationResponse<Origin>> BeginCreateWithHttpMessagesAsync(string originName, OriginParameters originProperties, string endpointName, string profileName, string resourceGroupName, Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Updates an existing CDN origin within an endpoint.
        /// </summary>
        /// <param name='originName'>
        /// Name of the origin. Must be unique within endpoint.
        /// </param>
        /// <param name='originProperties'>
        /// Origin properties
        /// </param>
        /// <param name='endpointName'>
        /// Name of the endpoint within the CDN profile.
        /// </param>
        /// <param name='profileName'>
        /// Name of the CDN profile within the resource group.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Name of the resource group within the Azure subscription.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        Task<AzureOperationResponse<Origin>> UpdateWithHttpMessagesAsync(string originName, OriginParameters originProperties, string endpointName, string profileName, string resourceGroupName, Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Updates an existing CDN origin within an endpoint.
        /// </summary>
        /// <param name='originName'>
        /// Name of the origin. Must be unique within endpoint.
        /// </param>
        /// <param name='originProperties'>
        /// Origin properties
        /// </param>
        /// <param name='endpointName'>
        /// Name of the endpoint within the CDN profile.
        /// </param>
        /// <param name='profileName'>
        /// Name of the CDN profile within the resource group.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Name of the resource group within the Azure subscription.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        Task<AzureOperationResponse<Origin>> BeginUpdateWithHttpMessagesAsync(string originName, OriginParameters originProperties, string endpointName, string profileName, string resourceGroupName, Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Deletes an existing CDN origin within an endpoint.
        /// </summary>
        /// <param name='originName'>
        /// Name of the origin. Must be unique within endpoint.
        /// </param>
        /// <param name='endpointName'>
        /// Name of the endpoint within the CDN profile.
        /// </param>
        /// <param name='profileName'>
        /// Name of the CDN profile within the resource group.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Name of the resource group within the Azure subscription.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        Task<AzureOperationResponse<Origin>> DeleteIfExistsWithHttpMessagesAsync(string originName, string endpointName, string profileName, string resourceGroupName, Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
        /// <summary>
        /// Deletes an existing CDN origin within an endpoint.
        /// </summary>
        /// <param name='originName'>
        /// Name of the origin. Must be unique within endpoint.
        /// </param>
        /// <param name='endpointName'>
        /// Name of the endpoint within the CDN profile.
        /// </param>
        /// <param name='profileName'>
        /// Name of the CDN profile within the resource group.
        /// </param>
        /// <param name='resourceGroupName'>
        /// Name of the resource group within the Azure subscription.
        /// </param>
        /// <param name='customHeaders'>
        /// The headers that will be added to request.
        /// </param>
        /// <param name='cancellationToken'>
        /// The cancellation token.
        /// </param>
        Task<AzureOperationResponse<Origin>> BeginDeleteIfExistsWithHttpMessagesAsync(string originName, string endpointName, string profileName, string resourceGroupName, Dictionary<string, List<string>> customHeaders = null, CancellationToken cancellationToken = default(CancellationToken));
    }
}
